@@ -1,5 +1,5 @@
 /* Rational - Rational number class with overflow detection
-   Copyright (C) 2012-2022 Antonio Diaz Diaz.
+   Copyright (C) 2012-2024 Antonio Diaz Diaz.
 
    This program is free software. Redistribution and use in source and
    binary forms, with or without modification, are permitted provided
@@ -18,9 +18,9 @@
 */
 /*
    Exit status: 0 for a normal exit, 1 for environmental problems
-   (file not found, invalid flags, I/O errors, etc), 2 to indicate a
-   corrupt or invalid input file, 3 for an internal consistency error
-   (e.g., bug) which caused rational to panic.
+   (file not found, invalid command-line options, I/O errors, etc), 2 to
+   indicate a corrupt or invalid input file, 3 for an internal consistency
+   error (e.g., bug) which caused rational to panic.
 */
 
 #include <cctype>
@@ -38,7 +38,7 @@
 namespace {
 
 const char * const program_name = "rational";
-const char * const program_year = "2022";
+const char * const program_year = "2024";
 const char * invocation_name = program_name;		// default value
 
 
@@ -46,13 +46,13 @@ void show_help()
   {
   std::printf( "Rational is a C++ class implementing rational numbers of limited size with\n"
                "overflow detection. Both numerator and denominator are of type 'int'.\n"
-               "\nRational allows you to perform exact calculations and verify at any time if\n"
-               "overflow has occurred. Rational does not throw exceptions.\n"
+               "\nRational allows you to perform exact calculations and check at any time\n"
+               "whether overflow has occurred. Rational does not throw exceptions.\n"
                "\nIn case of domain error or overflow, denominator is set to 0 and numerator\n"
                "is set to >0, <0, or 0, meaning +INF, -INF, and NAN respectively. This error\n"
                "condition can be tested with the function 'error', and can only be cleared\n"
                "by assigning a new value to the Rational.\n"
-               "\nWhile in error state, arithmetic operators become no ops and relational\n"
+               "\nWhile in error state, arithmetic operators become no-ops and relational\n"
                "operators return false, except '!=', which returns true. This is done to\n"
                "preserve the error condition which triggered the error, and because the\n"
                "error value is considered to be out of domain and therefore is different\n"
@@ -130,7 +130,7 @@ void approximate_golden_ratio()
     int i = 2;
     for( ; i < tlen; ++i )
       {
-      rest *= 10; const int dif = '0' + ( rest / d ) - target[i]; rest %= d;
+      rest *= 10; const int dif = ( rest / d ) + '0' - target[i]; rest %= d;
       if( dif != 0 ) break;
       }
     if( i > max_len )
@@ -149,24 +149,33 @@ void approximate_golden_ratio()
   }
 
 
+/* return >0 if n1/d1 > n2/d2, <0 if n1/d1 < n2/d2, 0 if n1/d1 == n2/d2
+   d1 and d2 must have the same sign */
+inline long long compare_fraction( const int n1, const int d1,
+                                   const int n2, const int d2 )
+  { return (long long)n1 * d2 - (long long)n2 * d1; }
+
+
 void approximate_pi()
   {
   const std::string target( "3.141592653589793238462643383279502884197" );
   const int tlen = target.size();
   int max_len = 1;		// number of chars identical to target
-  int n = 3, d = 1, dif = 0;
+  int n = 3, d = 1, dif = 0;	// initialize n/d to the integer part of target
+  int nmin = 29, dmin = 10;	// lower limit of search (2.9)
+  int nmax = 32, dmax = 10;	// upper limit of search (3.2)
 
   std::printf( "target = %s\n", target.c_str() );
   std::fputs( "correct\ndecimals                  value\n", stdout );
   while( n <= INT_MAX - 3 )
     {
-    const int q = n / d;
-    if( q != 3 ) { if( q < 3 ) ++n; else { n += 3; ++d; } continue; }
+    if( compare_fraction( n, d, nmax, dmax ) >= 0 ) { n += 3; ++d; continue; }
+    if( compare_fraction( n, d, nmin, dmin ) <= 0 ) { ++n; continue; }
     long long rest = n % d;
     int i = 2;
     for( ; i < tlen; ++i )
       {
-      rest *= 10; dif = '0' + ( rest / d ) - target[i]; rest %= d;
+      rest *= 10; dif = ( rest / d ) + '0' - target[i]; rest %= d;
       if( dif != 0 ) break;
       }
     if( i > max_len )
@@ -179,7 +188,8 @@ void approximate_pi()
       std::fflush( stdout );
       if( max_len >= tlen ) break;
       }
-    if( dif < 0 ) ++n; else { n += 3; ++d; }
+    if( dif < 0 ) { nmin = n; dmin = d; ++n; }	// increase lower limit
+    else { nmax = n; dmax = d; n += 3; ++d; }	// decrease upper limit
     }
   }
 
@@ -648,7 +658,7 @@ int main( const int argc, const char * const argv[] )
                 break;
       case 'v': verbose = true; break;
       case 'V': show_version(); return 0;
-      default : internal_error( "uncaught option." );
+      default: internal_error( "uncaught option." );
       }
     } // end process options
 
